@@ -341,8 +341,19 @@ static int cgi_demux_response(server *srv, handler_ctx *hctx) {
 
 	while(1) {
 		int n;
+		int toread;
+
+#if defined(__WIN32)
+		buffer_prepare_copy(hctx->response, 4 * 1024);
+#else
+		if (ioctl(con->fd, FIONREAD, &toread) || toread == 0 || toread <= 4*1024) {
+			buffer_prepare_copy(hctx->response, 4 * 1024);
+		} else {
+			if (toread > MAX_READ_LIMIT) toread = MAX_READ_LIMIT;
+			buffer_prepare_copy(hctx->response, toread + 1);
+		}
+#endif
 
-		buffer_prepare_copy(hctx->response, 1024);
 		if (-1 == (n = read(hctx->fd, hctx->response->ptr, hctx->response->size - 1))) {
 			if (errno == EAGAIN || errno == EINTR) {
 				/* would block, wait for signal */
@@ -917,7 +928,7 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 		cgi_env_add(&env, CONST_STR_LEN("CONTENT_LENGTH"), buf, strlen(buf));
 		cgi_env_add(&env, CONST_STR_LEN("SCRIPT_FILENAME"), CONST_BUF_LEN(con->physical.path));
 		cgi_env_add(&env, CONST_STR_LEN("SCRIPT_NAME"), CONST_BUF_LEN(con->uri.path));
-		cgi_env_add(&env, CONST_STR_LEN("DOCUMENT_ROOT"), CONST_BUF_LEN(con->physical.doc_root));
+		cgi_env_add(&env, CONST_STR_LEN("DOCUMENT_ROOT"), CONST_BUF_LEN(con->physical.basedir));
 
 		/* for valgrind */
 		if (NULL != (s = getenv("LD_PRELOAD"))) {
