@@ -306,6 +306,8 @@ static void server_free(server *srv) {
 			buffer_free(s->ssl_pemfile);
 			buffer_free(s->ssl_ca_file);
 			buffer_free(s->ssl_cipher_list);
+			buffer_free(s->ssl_dh_file);
+			buffer_free(s->ssl_ec_curve);
 			buffer_free(s->error_handler);
 			buffer_free(s->errorfile_prefix);
 			array_free(s->mimetypes);
@@ -391,6 +393,11 @@ static void show_features (void) {
 #else
       "\t- /dev/poll (Solaris)\n"
 #endif
+#ifdef USE_SOLARIS_PORT
+      "\t+ eventports (Solaris)\n"
+#else
+      "\t- eventports (Solaris)\n"
+#endif
 #ifdef USE_FREEBSD_KQUEUE
       "\t+ kqueue (FreeBSD)\n"
 #else
