@@ -6,6 +6,8 @@ use IO::Socket;
 use Test::More;
 use Socket;
 use Cwd 'abs_path';
+use POSIX qw(:sys_wait_h dup2);
+use Errno qw(EADDRINUSE);
 
 sub mtime {
 	my $file = shift;
@@ -30,14 +32,17 @@ sub new {
 
 
 	if (mtime($self->{BASEDIR}.'/src/lighttpd') > mtime($self->{BASEDIR}.'/build/lighttpd')) {
-		$self->{LIGHTTPD_PATH} = $self->{BASEDIR}.'/src/lighttpd';
-		$self->{MODULES_PATH} = $self->{BASEDIR}.'/src/.libs';
+		$self->{BINDIR} = $self->{BASEDIR}.'/src';
+		if (mtime($self->{BASEDIR}.'/src/.libs')) {
+			$self->{MODULES_PATH} = $self->{BASEDIR}.'/src/.libs';
+		} else {
+			$self->{MODULES_PATH} = $self->{BASEDIR}.'/src';
+		}
 	} else {
-		$self->{LIGHTTPD_PATH} = $self->{BASEDIR}.'/build/lighttpd';
+		$self->{BINDIR} = $self->{BASEDIR}.'/build';
 		$self->{MODULES_PATH} = $self->{BASEDIR}.'/build';
 	}
-	$self->{LIGHTTPD_PIDFILE} = $self->{TESTDIR}.'/tmp/lighttpd/lighttpd.pid';
-	$self->{PIDOF_PIDFILE} = $self->{TESTDIR}.'/tmp/lighttpd/pidof.pid';
+	$self->{LIGHTTPD_PATH} = $self->{BINDIR}.'/lighttpd';
 	$self->{PORT} = 2048;
 
 	my ($name, $aliases, $addrtype, $net) = gethostbyaddr(inet_aton("127.0.0.1"), AF_INET);
@@ -66,58 +71,79 @@ sub listening_on {
 sub stop_proc {
 	my $self = shift;
 
-	open F, $self->{LIGHTTPD_PIDFILE} or return -1;
-	my $pid = <F>;
-	close F;
-
-	if (defined $pid) {
-		kill('TERM',$pid) or return -1;
-		select(undef, undef, undef, 0.1);
+	my $pid = $self->{LIGHTTPD_PID};
+	if (defined $pid && $pid != -1) {
+		kill('TERM', $pid) or return -1;
+		return -1 if ($pid != waitpid($pid, 0));
+	} else {
+		diag("\nProcess not started, nothing to stop");
+		return -1;
 	}
 
 	return 0;
 }
 
+sub wait_for_port_with_proc {
+	my $self = shift;
+	my $port = shift;
+	my $child = shift;
+	my $timeout = 5*10; # 5 secs, select waits 0.1 s
+
+	while (0 == $self->listening_on($port)) {
+		select(undef, undef, undef, 0.1);
+		$timeout--;
+
+		# the process is gone, we failed
+		if (0 != waitpid($child, WNOHANG)) {
+			return -1;
+		}
+		if (0 >= $timeout) {
+			diag("\nTimeout while trying to connect; killing child");
+			kill('TERM', $child);
+			return -1;
+		}
+	}
+
+	return 0;
+}
 
 sub start_proc {
 	my $self = shift;
 	# kill old proc if necessary
-	$self->stop_proc;
+	#$self->stop_proc;
 
 	# pre-process configfile if necessary
 	#
 
 	$ENV{'SRCDIR'} = $self->{BASEDIR}.'/tests';
+	$ENV{'PORT'} = $self->{PORT};
 
-	unlink($self->{LIGHTTPD_PIDFILE});
+	my $cmdline = $self->{LIGHTTPD_PATH}." -D -f ".$self->{SRCDIR}."/".$self->{CONFIGFILE}." -m ".$self->{MODULES_PATH};
 	if (defined $ENV{"TRACEME"} && $ENV{"TRACEME"} eq 'strace') {
-		system("strace -tt -s 512 -o strace ".$self->{LIGHTTPD_PATH}." -D -f ".$self->{SRCDIR}."/".$self->{CONFIGFILE}." -m ".$self->{MODULES_PATH}." &");
+		$cmdline = "strace -tt -s 512 -o strace ".$cmdline;
 	} elsif (defined $ENV{"TRACEME"} && $ENV{"TRACEME"} eq 'truss') {
-		system("/usr/dtrctkit/bin/dtruss -d -e ".$self->{LIGHTTPD_PATH}." -D -f ".$self->{SRCDIR}."/".$self->{CONFIGFILE}." -m ".$self->{MODULES_PATH}." 2> strace &");
+		$cmdline = "truss -a -l -w all -v all -o strace ".$cmdline;
+	} elsif (defined $ENV{"TRACEME"} && $ENV{"TRACEME"} eq 'gdb') {
+		$cmdline = "gdb --batch --ex 'run' --ex 'bt full' --args ".$cmdline." > gdb.out";
 	} elsif (defined $ENV{"TRACEME"} && $ENV{"TRACEME"} eq 'valgrind') {
-		system("valgrind --tool=memcheck --show-reachable=yes --leak-check=yes --log-file=valgrind ".$self->{LIGHTTPD_PATH}." -D -f ".$self->{SRCDIR}."/".$self->{CONFIGFILE}." -m ".$self->{MODULES_PATH}." &");
-	} else {
-		system($self->{LIGHTTPD_PATH}." -f ".$self->{SRCDIR}."/".$self->{CONFIGFILE}." -m ".$self->{MODULES_PATH});
+		$cmdline = "valgrind --tool=memcheck --show-reachable=yes --leak-check=yes --log-file=valgrind ".$cmdline;
+	}
+	# diag("\nstarting lighttpd at :".$self->{PORT}.", cmdline: ".$cmdline );
+	my $child = fork();
+	if (not defined $child) {
+		diag("\nFork failed");
+		return -1;
+	}
+	if ($child == 0) {
+		exec $cmdline or die($?);
 	}
 
-	select(undef, undef, undef, 0.1);
-	if (not -e $self->{LIGHTTPD_PIDFILE} or 0 == kill 0, `cat $self->{LIGHTTPD_PIDFILE}`) {
-		select(undef, undef, undef, 2);	
-	}
-
-	unlink($self->{TESTDIR}."/tmp/cfg.file");
-
-	# no pidfile, we failed
-	if (not -e $self->{LIGHTTPD_PIDFILE}) {
-		diag(sprintf('Could not find pidfile: %s', $self->{LIGHTTPD_PIDFILE}));
+	if (0 != $self->wait_for_port_with_proc($self->{PORT}, $child)) {
+		diag(sprintf('\nThe process %i is not up', $child));
 		return -1;
 	}
 
-	# the process is gone, we failed
-	if (0 == kill 0, `cat $self->{LIGHTTPD_PIDFILE}`) {
-		diag(sprintf('the process referenced by %s is not up', $self->{LIGHTTPD_PIDFILE}));
-		return -1;
-	}
+	$self->{LIGHTTPD_PID} = $child;
 
 	0;
 }
@@ -131,6 +157,8 @@ sub handle_http {
 
 	my @request = $t->{REQUEST};
 	my @response = $t->{RESPONSE};
+	my $slow = defined $t->{SLOWREQUEST};
+	my $is_debug = $ENV{"TRACE_HTTP"};
 
 	my $remote = 
  	  IO::Socket::INET->new(Proto    => "tcp",
@@ -138,26 +166,56 @@ sub handle_http {
 				PeerPort => $self->{PORT});
 
 	if (not defined $remote) {
-		diag("connect failed: $!");
+		diag("\nconnect failed: $!");
 	       	return -1;
 	}
 
 	$remote->autoflush(1);
 
-	foreach(@request) {
-		# pipeline requests
-		s/\r//g;
-		s/\n/$EOL/g;
+	if (!$slow) {
+		diag("\nsending request header to ".$host.":".$self->{PORT}) if $is_debug;
+		foreach(@request) {
+			# pipeline requests
+			s/\r//g;
+			s/\n/$EOL/g;
 
-		print $remote $_.$BLANK;	
+			print $remote $_.$BLANK;
+			diag("\n<< ".$_) if $is_debug;
+		}
+		shutdown($remote, 1); # I've stopped writing data
+	} else {
+		diag("\nsending request header to ".$host.":".$self->{PORT}) if $is_debug;
+		foreach(@request) {
+			# pipeline requests
+			chomp;
+			s/\r//g;
+			s/\n/$EOL/g;
+
+			print $remote $_;
+			diag("<< ".$_."\n") if $is_debug;
+			select(undef, undef, undef, 0.1);
+			print $remote "\015";
+			select(undef, undef, undef, 0.1);
+			print $remote "\012";
+			select(undef, undef, undef, 0.1);
+			print $remote "\015";
+			select(undef, undef, undef, 0.1);
+			print $remote "\012";
+			select(undef, undef, undef, 0.1);
+		}
+	
 	}
+	diag("\n... done") if $is_debug;
 
 	my $lines = "";
 
+	diag("\nreceiving response") if $is_debug;
 	# read everything
 	while(<$remote>) {
 		$lines .= $_;
+		diag(">> ".$_) if $is_debug;
 	}
+	diag("\n... done") if $is_debug;
 	
 	close $remote;
 
@@ -175,7 +233,7 @@ sub handle_http {
 			(my $line, $lines) = split($EOL, $lines, 2);
 
 			# header finished
-			last if(length($line) == 0);
+			last if(!defined $line or length($line) == 0);
 
 			if ($ln == 0) {
 				# response header
@@ -187,18 +245,24 @@ sub handle_http {
 					(my $h = $1) =~ tr/[A-Z]/[a-z]/;
 
 					if (defined $resp_hdr{$h}) {
-						diag(sprintf("header %s is duplicated: %s and %s\n",
-						             $h, $resp_hdr{$h}, $2));
+# 						diag(sprintf("\nheader '%s' is duplicated: '%s' and '%s'\n",
+# 						             $h, $resp_hdr{$h}, $2));
+						$resp_hdr{$h} .= ', '.$2;
 					} else {
 						$resp_hdr{$h} = $2;
 					}
 				} else {
-					diag(sprintf("unexpected line '$line'\n"));
+					diag(sprintf("\nunexpected line '%s'", $line));
 					return -1;
 				}
 			}
 		}
 
+		if (not defined($resp_line)) {
+			diag(sprintf("\nempty response"));
+			return -1;
+		}
+
 		$t->{etag} = $resp_hdr{'etag'};
 		$t->{date} = $resp_hdr{'date'};
 
@@ -219,33 +283,34 @@ sub handle_http {
 		# check conditions
 		if ($resp_line =~ /^(HTTP\/1\.[01]) ([0-9]{3}) .+$/) {
 			if ($href->{'HTTP-Protocol'} ne $1) {
-				diag(sprintf("proto failed: expected '%s', got '%s'\n", $href->{'HTTP-Protocol'}, $1));
+				diag(sprintf("\nproto failed: expected '%s', got '%s'", $href->{'HTTP-Protocol'}, $1));
 				return -1;
 			}
 			if ($href->{'HTTP-Status'} ne $2) {
-				diag(sprintf("status failed: expected '%s', got '%s'\n", $href->{'HTTP-Status'}, $2));
+				diag(sprintf("\nstatus failed: expected '%s', got '%s'", $href->{'HTTP-Status'}, $2));
 				return -1;
 			}
 		} else {
-			diag(sprintf("unexpected resp_line '$resp_line'\n"));
+			diag(sprintf("\nunexpected resp_line '%s'", $resp_line));
 			return -1;
 		}
 
 		if (defined $href->{'HTTP-Content'}) {
 			$resp_body = "" unless defined $resp_body;
 			if ($href->{'HTTP-Content'} ne $resp_body) {
-				diag(sprintf("body failed: expected '%s', got '%s'\n", $href->{'HTTP-Content'}, $resp_body));
+				diag(sprintf("\nbody failed: expected '%s', got '%s'", $href->{'HTTP-Content'}, $resp_body));
 				return -1;
 			}
-		} elsif (defined $href->{'-HTTP-Content'}) {
+		}
+		
+		if (defined $href->{'-HTTP-Content'}) {
 			if (defined $resp_body && $resp_body ne '') {
-				diag(sprintf("body failed: expected empty body, got '%s'\n", $resp_body));
+				diag(sprintf("\nbody failed: expected empty body, got '%s'", $resp_body));
 				return -1;
 			}
 		}
 
 		foreach (keys %{ $href }) {
-			## filter special keys
 			next if $_ eq 'HTTP-Protocol';
 			next if $_ eq 'HTTP-Status';
 			next if $_ eq 'HTTP-Content';
@@ -257,7 +322,6 @@ sub handle_http {
 			my $key_inverted = 0;
 
 			if (substr($k, 0, 1) eq '+') {
-				## the key has to exist, but the value is ignored
 				$k = substr($k, 1);
 				$verify_value = 0;
 			} elsif (substr($k, 0, 1) eq '-') {
@@ -265,16 +329,16 @@ sub handle_http {
 				$k = substr($k, 1);
 				$key_inverted = 1;
 				$verify_value = 0; ## skip the value check
-			}
+                        }
 
 			if ($key_inverted) {
 				if (defined $resp_hdr{$k}) {
-					diag(sprintf("required header '%s' is missing\n", $k));
+					diag(sprintf("\nheader '%s' MUST not be set", $k));
 					return -1;
 				}
 			} else {
 				if (not defined $resp_hdr{$k}) {
-					diag(sprintf("required header '%s' is missing\n", $k));
+					diag(sprintf("\nrequired header '%s' is missing", $k));
 					return -1;
 				}
 			}
@@ -282,12 +346,12 @@ sub handle_http {
 			if ($verify_value) {
 				if ($href->{$_} =~ /^\/(.+)\/$/) {
 					if ($resp_hdr{$k} !~ /$1/) {
-						diag(sprintf("response-header failed: expected '%s', got '%s', regex: %s\n", 
+						diag(sprintf("\nresponse-header failed: expected '%s', got '%s', regex: %s", 
 					             $href->{$_}, $resp_hdr{$k}, $1));
 						return -1;
 					}
 				} elsif ($href->{$_} ne $resp_hdr{$k}) {
-					diag(sprintf("response-header failed: expected '%s', got '%s'\n", 
+					diag(sprintf("\nresponse-header failed: expected '%s', got '%s'",
 					     $href->{$_}, $resp_hdr{$k}));
 					return -1;
 				}
@@ -297,12 +361,44 @@ sub handle_http {
 
 	# we should have sucked up everything
 	if (defined $lines) {
-		diag(sprintf("unexpected lines '$lines'\n"));
+		diag(sprintf("\nunexpected lines '%s'", $lines));
 		return -1;
 	}
 
 	return 0;
 }
-    
-1;
 
+sub spawnfcgi {
+	my ($self, $binary, $port) = @_;
+	my $child = fork();
+	if (not defined $child) {
+		diag("\nCouldn't fork");
+		return -1;
+	}
+	if ($child == 0) {
+		my $iaddr   = inet_aton('localhost') || die "no host: localhost";
+		my $proto   = getprotobyname('tcp');
+		socket(SOCK, PF_INET, SOCK_STREAM, $proto) || die "socket: $!";
+		setsockopt(SOCK, SOL_SOCKET, SO_REUSEADDR, pack("l", 1)) || die "setsockopt: $!";
+		bind(SOCK, sockaddr_in($port, $iaddr)) || die "bind: $!";
+		listen(SOCK, 1024) || die "listen: $!";
+		dup2(fileno(SOCK), 0) || die "dup2: $!";
+		exec $binary or die($?);
+	} else {
+		if (0 != $self->wait_for_port_with_proc($port, $child)) {
+			diag(sprintf("\nThe process %i is not up (port %i, %s)", $child, $port, $binary));
+			return -1;
+		}
+		return $child;
+	}
+}
+
+sub endspawnfcgi {
+	my ($self, $pid) = @_;
+	return -1 if (-1 == $pid);
+	kill(2, $pid);
+	waitpid($pid, 0);
+	return 0;
+}
+
+1;
